@@ -112,6 +112,8 @@ int create_client_dup( char ** server_list, gearman_client_st *client ) {
         x++;
     }
 
+    current_client_dup = client;
+
     return GM_OK;
 }
 
@@ -150,6 +152,9 @@ int create_client( char ** server_list, gearman_client_st *client ) {
     }
     assert(x != 0);
 
+
+    current_client = client;
+
     return GM_OK;
 }
 
@@ -159,9 +164,22 @@ int add_job_to_queue( gearman_client_st *client, char ** server_list, char * que
     gearman_task_st *task = NULL;
     gearman_return_t ret1, ret2;
     char * crypted_data;
-    int size;
+    int size, free_uniq;
     struct timeval now;
 
+    /* check too long queue names */
+    if(strlen(queue) > 63) {
+        gm_log( GM_LOG_ERROR, "queue name too long: '%s'\n", queue );
+        return GM_ERROR;
+    }
+
+    /* cut off to long uniq ids */
+    free_uniq = 0;
+    if(uniq != NULL && strlen(uniq) > 63) {
+        uniq = strndup(uniq, 63);
+        free_uniq = 1;
+    }
+
     signal(SIGPIPE, SIG_IGN);
 
     gm_log( GM_LOG_TRACE, "add_job_to_queue(%s, %s, %d, %d, %d, %d)\n", queue, uniq, priority, retries, transport_mode, send_now );
@@ -248,6 +266,9 @@ int add_job_to_queue( gearman_client_st *client, char ** server_list, char * que
     /* reset error counter */
     mod_gm_con_errors = 0;
 
+    if(free_uniq)
+        free(uniq);
+
     gm_log( GM_LOG_TRACE, "add_job_to_queue() finished sucessfully: %d %d\n", ret1, ret2 );
     return GM_OK;
 }
