@@ -24,40 +24,55 @@
 #include "utils.h"
 #include "crypt.h"
 #include "base64.h"
+#include "gearman.h"
+#include "popenRWE.h"
 
 pid_t current_child_pid = 0;
+char temp_buffer1[GM_BUFFERSIZE];
+char temp_buffer2[GM_BUFFERSIZE];
 
 /* escapes newlines in a string */
-char *escape_newlines(char *rawbuf) {
+char *gm_escape_newlines(char *rawbuf, int trimmed) {
+    char *tmpbuf=NULL;
+    char *tmpbuf_dup=NULL;
     char *newbuf=NULL;
     register int x,y;
 
     if(rawbuf==NULL)
         return NULL;
 
+    tmpbuf     = strdup(rawbuf);
+    tmpbuf_dup = tmpbuf;
+    if ( trimmed == GM_ENABLED ) {
+        tmpbuf = trim(tmpbuf);
+    }
+
     /* allocate enough memory to escape all chars if necessary */
-    if((newbuf=malloc((strlen(rawbuf)*2)+1))==NULL)
+    if((newbuf=malloc((strlen(tmpbuf)*2)+1))==NULL)
         return NULL;
 
-    for(x=0,y=0;rawbuf[x]!=(char)'\x0';x++){
+    for(x=0,y=0;tmpbuf[x]!=(char)'\x0';x++){
 
         /* escape backslashes */
-        if(rawbuf[x]=='\\'){
+        if(tmpbuf[x]=='\\'){
             newbuf[y++]='\\';
             newbuf[y++]='\\';
         }
 
         /* escape newlines */
-        else if(rawbuf[x]=='\n'){
+        else if(tmpbuf[x]=='\n'){
             newbuf[y++]='\\';
             newbuf[y++]='n';
         }
 
         else
-            newbuf[y++]=rawbuf[x];
+            newbuf[y++]=tmpbuf[x];
     }
+
     newbuf[y]='\x0';
 
+    free(tmpbuf_dup);
+
     return newbuf;
 }
 
@@ -161,6 +176,8 @@ char *rtrim(char *s) {
     char *back;
     if(s == NULL)
         return NULL;
+    if(strlen(s) == 0)
+        return s;
     back = s + strlen(s);
     while(isspace(*--back));
     *(back+1) = '\0';
@@ -223,6 +240,9 @@ int set_default_options(mod_gm_opt_t *opt) {
     opt->max_jobs           = GM_DEFAULT_MAX_JOBS;
     opt->spawn_rate         = GM_DEFAULT_SPAWN_RATE;
     opt->identifier         = NULL;
+    opt->show_error_output  = GM_ENABLED;
+
+    opt->workaround_rc_25   = GM_DISABLED;
 
     opt->host               = NULL;
     opt->service            = NULL;
@@ -245,7 +265,7 @@ int set_default_options(mod_gm_opt_t *opt) {
     opt->local_servicegroups_num  = 0;
     for(i=0;i<=GM_LISTSIZE;i++)
         opt->local_servicegroups_list[i] = NULL;
-    for(i=0;i<=GM_NEBTYPESSIZE;i++) {
+    for(i=0;i<GM_NEBTYPESSIZE;i++) {
         mod_gm_exp_t *mod_gm_exp;
         mod_gm_exp              = malloc(sizeof(mod_gm_exp_t));
         mod_gm_exp->elem_number = 0;
@@ -288,7 +308,6 @@ int parse_yes_or_no(char*value, int dfl) {
 int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
     char *key;
     char *value;
-    char temp_buffer[GM_BUFFERSIZE];
 
     gm_log( GM_LOG_TRACE, "parse_args_line(%s, %d)\n", arg, recursion_level);
 
@@ -367,10 +386,23 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
     }
 
     /* active */
-    else if (   !strcmp( key, "active" ) ) {
+    else if ( !strcmp( key, "active" ) ) {
         opt->active = parse_yes_or_no(value, GM_ENABLED);
         return(GM_OK);
     }
+
+    /* show_error_output */
+    else if ( !strcmp( key, "show_error_output" ) ) {
+        opt->show_error_output = parse_yes_or_no(value, GM_ENABLED);
+        return(GM_OK);
+    }
+
+    /* workaround_rc_25 */
+    else if ( !strcmp( key, "workaround_rc_25" ) ) {
+        opt->workaround_rc_25 = parse_yes_or_no(value, GM_ENABLED);
+        return(GM_OK);
+    }
+
     else if ( value == NULL ) {
         gm_log( GM_LOG_ERROR, "unknown switch '%s'\n", key );
         return(GM_OK);
@@ -461,9 +493,10 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         string2timeval(value, &opt->finishtime);
     }
 
-    /* configfile */
+    /* configfile / includes */
     else if (   !strcmp( key, "config" )
              || !strcmp( key, "configfile" )
+             || !strcmp( key, "include" )
             ) {
         if(read_config_file(opt, value, ++recursion_level) != GM_OK) {
             recursion_level--;
@@ -561,16 +594,9 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
     else if ( !strcmp( key, "server" ) ) {
         char *servername;
         while ( (servername = strsep( &value, "," )) != NULL ) {
-            servername = trim(servername);
-            if ( strcmp( servername, "" ) ) {
-                if(strcspn(servername, ":") == 0) {
-                    temp_buffer[0]='\x0';
-                    snprintf( temp_buffer,sizeof( temp_buffer )-1, "localhost%s", servername);
-                    temp_buffer[sizeof( temp_buffer )-1]='\x0';
-                    opt->server_list[opt->server_num] = strdup(temp_buffer);
-                } else {
-                    opt->server_list[opt->server_num] = strdup(servername);
-                }
+            char * new_server = get_param_server(servername, opt->server_list, opt->server_num);
+            if(new_server != NULL) {
+                opt->server_list[opt->server_num] = new_server;
                 opt->server_num++;
             }
         }
@@ -580,9 +606,9 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
     else if ( !strcmp( key, "dupserver" ) ) {
         char *servername;
         while ( (servername = strsep( &value, "," )) != NULL ) {
-            servername = trim(servername);
-            if ( strcmp( servername, "" ) ) {
-                opt->dupserver_list[opt->dupserver_num] = strdup(servername);
+            char * new_server = get_param_server(servername, opt->dupserver_list, opt->dupserver_num);
+            if(new_server != NULL) {
+                opt->dupserver_list[opt->dupserver_num] = new_server;
                 opt->dupserver_num++;
             }
         }
@@ -596,9 +622,13 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         while ( (groupname = strsep( &value, "," )) != NULL ) {
             groupname = trim(groupname);
             if ( strcmp( groupname, "" ) ) {
-                opt->servicegroups_list[opt->servicegroups_num] = strdup(groupname);
-                opt->servicegroups_num++;
-                opt->set_queues_by_hand++;
+                if(strlen(groupname) > 50) {
+                    gm_log( GM_LOG_ERROR, "servicegroup name '%s' is too long, please use a maximum of 50 characters\n", groupname );
+                } else {
+                    opt->servicegroups_list[opt->servicegroups_num] = strdup(groupname);
+                    opt->servicegroups_num++;
+                    opt->set_queues_by_hand++;
+                }
             }
         }
     }
@@ -610,9 +640,13 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         while ( (groupname = strsep( &value, "," )) != NULL ) {
             groupname = trim(groupname);
             if ( strcmp( groupname, "" ) ) {
-                opt->hostgroups_list[opt->hostgroups_num] = strdup(groupname);
-                opt->hostgroups_num++;
-                opt->set_queues_by_hand++;
+                if(strlen(groupname) > 50) {
+                    gm_log( GM_LOG_ERROR, "hostgroup name '%s' is too long, please use a maximum of 50 characters\n", groupname );
+                } else {
+                    opt->hostgroups_list[opt->hostgroups_num] = strdup(groupname);
+                    opt->hostgroups_num++;
+                    opt->set_queues_by_hand++;
+                }
             }
         }
     }
@@ -653,31 +687,35 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         char *return_code   = strsep( &value, ":" );
         int return_code_num = atoi(return_code);
         char *callbacks     = strsep( &value, ":" );
-        while ( (callback = strsep( &callbacks, "," )) != NULL ) {
-            int callback_num = atoi(trim(callback));
-            if(index(callback, 'N') != NULL) {
-                callback_num = -1;
-                /* get neb callback number by name */
-                int i;
-                for(i=0;i<=GM_NEBTYPESSIZE;i++) {
-                    char * type = nebcallback2str(i);
-                    if(!strcmp(type, callback)) {
-                        callback_num = i;
+        if(strlen(export_queue) > 50) {
+            gm_log( GM_LOG_ERROR, "export queue name '%s' is too long, please use a maximum of 50 characters\n", export_queue );
+        } else {
+            while ( (callback = strsep( &callbacks, "," )) != NULL ) {
+                int callback_num = atoi(trim(callback));
+                if(index(callback, 'N') != NULL) {
+                    callback_num = -1;
+                    /* get neb callback number by name */
+                    int i;
+                    for(i=0;i<GM_NEBTYPESSIZE;i++) {
+                        char * type = nebcallback2str(i);
+                        if(!strcmp(type, callback)) {
+                            callback_num = i;
+                        }
+                        free(type);
+                    }
+                    if(callback_num == -1) {
+                        gm_log( GM_LOG_ERROR, "unknown nebcallback : %s\n", callback);
+                        continue;
                     }
-                    free(type);
                 }
-                if(callback_num == -1) {
-                    gm_log( GM_LOG_ERROR, "unknown nebcallback : %s\n", callback);
-                    continue;
-                }
-            }
 
-            int number = opt->exports[callback_num]->elem_number;
-            opt->exports[callback_num]->name[number]        = strdup(export_queue);
-            opt->exports[callback_num]->return_code[number] = return_code_num;
-            opt->exports[callback_num]->elem_number++;
+                int number = opt->exports[callback_num]->elem_number;
+                opt->exports[callback_num]->name[number]        = strdup(export_queue);
+                opt->exports[callback_num]->return_code[number] = return_code_num;
+                opt->exports[callback_num]->elem_number++;
+            }
+            opt->exports_count++;
         }
-        opt->exports_count++;
     }
 
     else {
@@ -804,7 +842,7 @@ void dumpconfig(mod_gm_opt_t *opt, int mode) {
         for(i=0;i<opt->local_servicegroups_num;i++)
             gm_log( GM_LOG_DEBUG, "local_servicegroups:      %s\n", opt->local_servicegroups_list[i]);
         /* export queues*/
-        for(i=0;i<=GM_NEBTYPESSIZE;i++) {
+        for(i=0;i<GM_NEBTYPESSIZE;i++) {
             char * type = nebcallback2str(i);
             for(j=0;j<opt->exports[i]->elem_number;j++)
                 gm_log( GM_LOG_DEBUG, "export:              %-45s -> %s\n", type, opt->exports[i]->name[j]);
@@ -845,7 +883,7 @@ void mod_gm_free_opt(mod_gm_opt_t *opt) {
         free(opt->local_hostgroups_list[i]);
     for(i=0;i<opt->local_servicegroups_num;i++)
         free(opt->local_servicegroups_list[i]);
-    for(i=0;i<=GM_NEBTYPESSIZE;i++) {
+    for(i=0;i<GM_NEBTYPESSIZE;i++) {
         for(j=0;j<opt->exports[i]->elem_number;j++) {
           free(opt->exports[i]->name[j]);
         }
@@ -994,7 +1032,7 @@ long mod_gm_time_compare(struct timeval * tv1, struct timeval * tv2) {
 
 
 /* extract check result */
-char *extract_check_result(FILE *fp) {
+char *extract_check_result(FILE *fp, int trimmed) {
     int size;
     char buffer[GM_BUFFERSIZE] = "";
     char output[GM_BUFFERSIZE] = "";
@@ -1007,7 +1045,8 @@ char *extract_check_result(FILE *fp) {
         strncat(output, buffer, size);
         size -= strlen(buffer);
     }
-    return(escape_newlines(output));
+
+    return(gm_escape_newlines(output, trimmed));
 }
 
 
@@ -1057,17 +1096,27 @@ int parse_command_line(char *cmd, char *argv[MAX_CMD_ARGS]) {
 
 
 /* run a check */
-int run_check(char *processed_command, char **ret) {
+int run_check(char *processed_command, char **ret, char **err) {
     char *argv[MAX_CMD_ARGS];
     FILE *fp;
     pid_t pid;
-    int pipefds[2];
+    int pipe_stdout[2], pipe_stderr[2], pipe_rwe[3];
     int retval;
 
     /* check for check execution method (shell or execvp) */
     if(!strpbrk(processed_command,"!$^&*()~[]|{};<>?`\"'")) {
+        /* use the fast execvp when there are now shell characters */
         gm_log( GM_LOG_TRACE, "using execvp\n" );
-        if(pipe(pipefds)) {
+
+        parse_command_line(processed_command,argv);
+        if(!argv[0])
+            _exit(STATE_UNKNOWN);
+
+        if(pipe(pipe_stdout)) {
+            gm_log( GM_LOG_ERROR, "error creating pipe: %s\n", strerror(errno));
+            _exit(STATE_UNKNOWN);
+        }
+        if(pipe(pipe_stderr)) {
             gm_log( GM_LOG_ERROR, "error creating pipe: %s\n", strerror(errno));
             _exit(STATE_UNKNOWN);
         }
@@ -1076,14 +1125,17 @@ int run_check(char *processed_command, char **ret) {
             _exit(STATE_UNKNOWN);
         }
         else if(!pid){
-            if((dup2(pipefds[1],STDOUT_FILENO)<0)||(dup2(pipefds[1],STDERR_FILENO)<0)){
+            /* child process */
+            if((dup2(pipe_stdout[1],STDOUT_FILENO)<0)){
                 gm_log( GM_LOG_ERROR, "dup2 error\n");
                 _exit(STATE_UNKNOWN);
             }
-            close(pipefds[1]);
-            parse_command_line(processed_command,argv);
-            if(!argv[0])
+            if((dup2(pipe_stderr[1],STDERR_FILENO)<0)){
+                gm_log( GM_LOG_ERROR, "dup2 error\n");
                 _exit(STATE_UNKNOWN);
+            }
+            close(pipe_stdout[1]);
+            close(pipe_stderr[1]);
             current_child_pid = getpid();
             execvp(argv[0], argv);
             if(errno == 2)
@@ -1093,52 +1145,77 @@ int run_check(char *processed_command, char **ret) {
             _exit(STATE_UNKNOWN);
         }
 
-        /* prepare pipe reading */
-        close(pipefds[1]);
-        fp=fdopen(pipefds[0],"r");
+        /* parent */
+        /* prepare stdout pipe reading */
+        close(pipe_stdout[1]);
+        fp=fdopen(pipe_stdout[0],"r");
         if(!fp){
             gm_log( GM_LOG_ERROR, "fdopen error\n");
             _exit(STATE_UNKNOWN);
         }
-
-        /* extract check result */
-        *ret = extract_check_result(fp);
-
-        /* close the process */
+        *ret = extract_check_result(fp, GM_DISABLED);
         fclose(fp);
-        close(pipefds[0]);
+
+        /* prepare stderr pipe reading */
+        close(pipe_stderr[1]);
+        fp=fdopen(pipe_stderr[0],"r");
+        if(!fp){
+            gm_log( GM_LOG_ERROR, "fdopen error\n");
+            _exit(STATE_UNKNOWN);
+        }
+        *err = extract_check_result(fp, GM_ENABLED);
+        fclose(fp);
+
+        close(pipe_stdout[0]);
+        close(pipe_stderr[0]);
         if(waitpid(pid,&retval,0)!=pid)
             retval=-1;
     }
     else {
-        current_child_pid = getpid();
+        /* use the slower popen when there were shell characters */
         gm_log( GM_LOG_TRACE, "using popen\n" );
-        fp=popen(processed_command,"r");
-        if(fp==NULL)
-            _exit(STATE_UNKNOWN);
+        current_child_pid = getpid();
+        pid = popenRWE(pipe_rwe, processed_command);
 
         /* extract check result */
-        *ret = extract_check_result(fp);
+        fp=fdopen(pipe_rwe[1],"r");
+        if(!fp){
+            gm_log( GM_LOG_ERROR, "fdopen error\n");
+            _exit(STATE_UNKNOWN);
+        }
+        *ret = extract_check_result(fp, GM_DISABLED);
+        fclose(fp);
 
+        /* extract check stderr */
+        fp=fdopen(pipe_rwe[2],"r");
+        if(!fp){
+            gm_log( GM_LOG_ERROR, "fdopen error\n");
+            _exit(STATE_UNKNOWN);
+        }
+        *err = extract_check_result(fp, GM_ENABLED);
+        fclose(fp);
 
         /* close the process */
-        retval=pclose(fp);
-     }
+        retval=pcloseRWE(pid, pipe_rwe);
+    }
 
     return retval;
 }
 
 
 /* execute this command with given timeout */
-int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * hostname) {
-    int pdes[2];
+int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier) {
+    int pipe_stdout[2] , pipe_stderr[2];
     int return_code;
     int pclose_result;
-    char *plugin_output;
-    char buffer[GM_BUFFERSIZE];
+    char *plugin_output, *plugin_error;
+    char *bufdup;
+    char buffer[GM_BUFFERSIZE], buf_error[GM_BUFFERSIZE];
     sigset_t mask;
     struct timeval start_time,end_time;
-    pid_t pid = 0;
+    pid_t pid    = 0;
+    buffer[0]    = '\x0';
+    buf_error[0] = '\x0';
 
     gm_log( GM_LOG_TRACE, "execute_safe_command()\n" );
 
@@ -1149,8 +1226,10 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * hostname) {
 
     /* fork a child process */
     if(fork_exec == GM_ENABLED) {
-        if(pipe(pdes) != 0)
-            perror("pipe");
+        if(pipe(pipe_stdout) != 0)
+            perror("pipe stdout");
+        if(pipe(pipe_stderr) != 0)
+            perror("pipe stderr");
 
         pid=fork();
 
@@ -1173,31 +1252,40 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * hostname) {
         sigfillset(&mask);
         sigprocmask(SIG_UNBLOCK, &mask, NULL);
 
-        if( fork_exec == GM_ENABLED )
-            close(pdes[0]);
+        if( fork_exec == GM_ENABLED ) {
+            close(pipe_stdout[0]);
+            close(pipe_stderr[0]);
+        }
         signal(SIGALRM, check_alarm_handler);
         alarm(exec_job->timeout);
 
         /* run the plugin check command */
-        pclose_result = run_check(exec_job->command_line, &plugin_output);
+        pclose_result = run_check(exec_job->command_line, &plugin_output, &plugin_error);
         return_code   = pclose_result;
 
         if(fork_exec == GM_ENABLED) {
-            if(write(pdes[1], plugin_output, strlen(plugin_output)+1) <= 0)
+            if(write(pipe_stdout[1], plugin_output, strlen(plugin_output)+1) <= 0)
+                perror("write stdout");
+            if(write(pipe_stderr[1], plugin_error, strlen(plugin_error)+1) <= 0)
                 perror("write");
 
             if(pclose_result == -1) {
                 char error[GM_BUFFERSIZE];
-                snprintf(error, sizeof(error), "error on %s: %s", hostname, strerror(errno));
-                if(write(pdes[1], error, strlen(error)+1) <= 0)
+                snprintf(error, sizeof(error), "error on %s: %s", identifier, strerror(errno));
+                if(write(pipe_stdout[1], error, strlen(error)+1) <= 0)
                     perror("write");
             }
 
             return_code = real_exit_code(pclose_result);
+            free(plugin_output);
+            free(plugin_error);
             exit(return_code);
         }
         else {
-            snprintf( buffer, sizeof( buffer )-1, "%s", plugin_output );
+            snprintf( buffer,    sizeof( buffer )-1,    "%s", plugin_output );
+            snprintf( buf_error, sizeof( buf_error )-1, "%s", plugin_error  );
+            free(plugin_output);
+            free(plugin_error);
         }
     }
 
@@ -1207,12 +1295,15 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * hostname) {
         gm_log( GM_LOG_TRACE, "started check with pid: %d\n", pid);
 
         if( fork_exec == GM_ENABLED) {
-            close(pdes[1]);
+            close(pipe_stdout[1]);
+            close(pipe_stderr[1]);
 
             waitpid(pid, &return_code, 0);
             gm_log( GM_LOG_TRACE, "finished check from pid: %d with status: %d\n", pid, return_code);
             /* get all lines of plugin output */
-            if(read(pdes[0], buffer, sizeof(buffer)-1) < 0)
+            if(read(pipe_stdout[0], buffer, sizeof(buffer)-1) < 0)
+                perror("read");
+            if(read(pipe_stderr[0], buf_error, sizeof(buf_error)-1) < 0)
                 perror("read");
         }
         return_code = real_exit_code(return_code);
@@ -1220,32 +1311,44 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * hostname) {
         /* file not executable? */
         if(return_code == 126) {
             return_code = STATE_CRITICAL;
-            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of 126 is out of bounds. Make sure the plugin you're trying to run is executable. (worker: %s)", hostname);
+            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of 126 is out of bounds. Make sure the plugin you're trying to run is executable. (worker: %s)", identifier);
         }
         /* file not found errors? */
         else if(return_code == 127) {
             return_code = STATE_CRITICAL;
-            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of 127 is out of bounds. Make sure the plugin you're trying to run actually exists. (worker: %s)", hostname);
+            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of 127 is out of bounds. Make sure the plugin you're trying to run actually exists. (worker: %s)", identifier);
         }
         /* signaled */
         else if(return_code >= 128 && return_code < 144) {
             char * signame = nr2signal((int)(return_code-128));
-            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of %d is out of bounds. Plugin exited by signal %s. (worker: %s)\n%s", (int)(return_code), signame, hostname, buffer);
+            bufdup = strdup(buffer);
+            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of %d is out of bounds. Plugin exited by signal %s. (worker: %s)", (int)(return_code), signame, identifier);
+            if(strlen(bufdup) > 0) {
+                strncat(buffer, "\\n", (sizeof(buffer)-1));
+                strncat(buffer, bufdup, (sizeof(buffer)-1));
+            }
             return_code = STATE_CRITICAL;
+            free(bufdup);
             free(signame);
         }
         /* other error codes > 3 */
         else if(return_code > 3) {
             gm_log( GM_LOG_INFO, "check exited with exit code > 3. Exit: %d\n", (int)(return_code));
             gm_log( GM_LOG_INFO, "stdout: %s\n", buffer);
-            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of %d is out of bounds. (worker: %s)\n%s\n", (int)(return_code), hostname, buffer);
-            return_code = STATE_CRITICAL;
+            bufdup = strdup(buffer);
+            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of %d is out of bounds. (worker: %s)\\n%s", (int)(return_code), identifier, bufdup);
+            free(bufdup);
+            if(return_code != 25 && mod_gm_opt->workaround_rc_25 == GM_DISABLED) {
+                return_code = STATE_CRITICAL;
+            }
         }
 
         exec_job->output      = strdup(buffer);
+        exec_job->error       = strdup(buf_error);
         exec_job->return_code = return_code;
         if( fork_exec == GM_ENABLED) {
-            close(pdes[0]);
+            close(pipe_stdout[0]);
+            close(pipe_stderr[0]);
         }
     }
     alarm(0);
@@ -1261,9 +1364,10 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * hostname) {
         exec_job->return_code   = 2;
         exec_job->early_timeout = 1;
         if ( !strcmp( exec_job->type, "service" ) )
-            snprintf( buffer, sizeof( buffer ) -1, "(Service Check Timed Out On Worker: %s)", hostname);
+            snprintf( buffer, sizeof( buffer ) -1, "(Service Check Timed Out On Worker: %s)", identifier);
         if ( !strcmp( exec_job->type, "host" ) )
-            snprintf( buffer, sizeof( buffer ) -1, "(Host Check Timed Out On Worker: %s)", hostname);
+            snprintf( buffer, sizeof( buffer ) -1, "(Host Check Timed Out On Worker: %s)", identifier);
+        free(exec_job->output);
         exec_job->output = strdup( buffer );
     }
 
@@ -1280,6 +1384,7 @@ int set_default_job(gm_job_t *job, mod_gm_opt_t *opt) {
     job->result_queue        = NULL;
     job->command_line        = NULL;
     job->output              = NULL;
+    job->error               = NULL;
     job->exited_ok           = TRUE;
     job->scheduled_check     = TRUE;
     job->reschedule_check    = TRUE;
@@ -1302,6 +1407,7 @@ int free_job(gm_job_t *job) {
     free(job->result_queue);
     free(job->command_line);
     free(job->output);
+    free(job->error);
     free(job);
 
     return(GM_OK);
@@ -1315,6 +1421,10 @@ void check_alarm_handler(int sig) {
     pid_t pid = getpid();
 
     gm_log( GM_LOG_TRACE, "check_alarm_handler(%i)\n", sig );
+    if(current_job != NULL && mod_gm_opt->fork_on_exec == GM_DISABLED) {
+        send_timeout_result(current_job);
+        gearman_job_send_complete(current_gearman_job, NULL, 0);
+    }
 
     if(current_child_pid > 0) {
         gm_log( GM_LOG_TRACE, "send SIGINT to %d\n", current_child_pid);
@@ -1799,3 +1909,175 @@ void gm_log( int lvl, const char *text, ... ) {
 
     return;
 }
+
+/* extract server from string and check for duplicates */
+char * get_param_server(char * servername, char * server_list[GM_LISTSIZE], int server_num) {
+    char temp_buffer[GM_BUFFERSIZE];
+    char * new_server;
+    int i;
+    servername = trim(servername);
+
+    if ( ! strcmp( servername, "" ) ) {
+        return NULL;
+    }
+
+    if(strcspn(servername, ":") == 0) {
+        temp_buffer[0]='\x0';
+        snprintf( temp_buffer,sizeof( temp_buffer )-1, "localhost%s", servername);
+        temp_buffer[sizeof( temp_buffer )-1]='\x0';
+        new_server = strdup(temp_buffer);
+    } else {
+        new_server = strdup(servername);
+    }
+
+    // check for duplicates
+    for(i=0;i<server_num;i++) {
+        if ( ! strcmp( new_server, server_list[i] ) ) {
+            gm_log( GM_LOG_ERROR, "duplicate definition of server: %s\n", new_server);
+            free(new_server);
+            return NULL;
+        }
+    }
+
+    return new_server;
+}
+
+void send_timeout_result(gm_job_t * exec_job) {
+    struct timeval end_time;
+    char buffer[GM_BUFFERSIZE];
+    buffer[0] = '\x0';
+
+    gm_log( GM_LOG_TRACE, "send_timeout_result()\n");
+
+    gettimeofday(&end_time, NULL);
+    exec_job->finish_time = end_time;
+
+    exec_job->return_code   = 2;
+    exec_job->early_timeout = 1;
+    if ( !strcmp( exec_job->type, "service" ) )
+        snprintf( buffer, sizeof( buffer ) -1, "(Service Check Timed Out On Worker: %s)\n", mod_gm_opt->identifier);
+    if ( !strcmp( exec_job->type, "host" ) )
+        snprintf( buffer, sizeof( buffer ) -1, "(Host Check Timed Out On Worker: %s)\n", mod_gm_opt->identifier);
+    free(exec_job->output);
+    exec_job->output = strdup( buffer );
+
+    send_result_back(exec_job);
+
+    return;
+}
+
+
+/* send results back */
+void send_result_back(gm_job_t * exec_job) {
+    gm_log( GM_LOG_TRACE, "send_result_back()\n" );
+
+    if(exec_job->result_queue == NULL) {
+        return;
+    }
+    if(exec_job->output == NULL) {
+        return;
+    }
+
+    /* workaround for rc 25 bug
+     * duplicate jobs from gearmand result in exit code 25 of plugins
+     * because they are executed twice and get killed because of using
+     * the same ressource.
+     * Sending results (when exit code is 25 ) will be skipped with this
+     * enabled.
+     */
+    if( exec_job->return_code == 25 && mod_gm_opt->workaround_rc_25 == GM_ENABLED ) {
+        return;
+    }
+
+    gm_log( GM_LOG_TRACE, "queue: %s\n", exec_job->result_queue );
+    temp_buffer1[0]='\x0';
+    snprintf( temp_buffer1, sizeof( temp_buffer1 )-1, "host_name=%s\ncore_start_time=%i.%i\nstart_time=%i.%i\nfinish_time=%i.%i\nlatency=%f\nreturn_code=%i\nexited_ok=%i\n",
+              exec_job->host_name,
+              ( int )exec_job->core_start_time.tv_sec,
+              ( int )exec_job->core_start_time.tv_usec,
+              ( int )exec_job->start_time.tv_sec,
+              ( int )exec_job->start_time.tv_usec,
+              ( int )exec_job->finish_time.tv_sec,
+              ( int )exec_job->finish_time.tv_usec,
+              exec_job->latency,
+              exec_job->return_code,
+              exec_job->exited_ok
+            );
+    temp_buffer1[sizeof( temp_buffer1 )-1]='\x0';
+
+    if(exec_job->service_description != NULL) {
+        temp_buffer2[0]='\x0';
+        strncat(temp_buffer2, "service_description=", (sizeof(temp_buffer2)-1));
+        strncat(temp_buffer2, exec_job->service_description, (sizeof(temp_buffer2)-1));
+        strncat(temp_buffer2, "\n", (sizeof(temp_buffer2)-1));
+
+        strncat(temp_buffer1, temp_buffer2, (sizeof(temp_buffer1)-1));
+    }
+    temp_buffer1[sizeof( temp_buffer1 )-1]='\x0';
+
+    if(exec_job->output != NULL) {
+        temp_buffer2[0]='\x0';
+        strncat(temp_buffer2, "output=", (sizeof(temp_buffer2)-1));
+        if(mod_gm_opt->debug_result) {
+            strncat(temp_buffer2, "(", (sizeof(temp_buffer2)-1));
+            strncat(temp_buffer2, hostname, (sizeof(temp_buffer2)-1));
+            strncat(temp_buffer2, ") - ", (sizeof(temp_buffer2)-1));
+        }
+        strncat(temp_buffer2, exec_job->output, (sizeof(temp_buffer2)-1));
+        if(mod_gm_opt->show_error_output && strlen(exec_job->error) > 0) {
+            if(strlen(exec_job->output) > 0)
+                strncat(temp_buffer2, "\\n", (sizeof(temp_buffer2)-1));
+            strncat(temp_buffer2, "[", (sizeof(temp_buffer2)-1));
+            strncat(temp_buffer2, exec_job->error, (sizeof(temp_buffer2)-1));
+            strncat(temp_buffer2, "] ", (sizeof(temp_buffer2)-1));
+        }
+        strncat(temp_buffer2, "\n\n\n", (sizeof(temp_buffer2)-1));
+        strncat(temp_buffer1, temp_buffer2, (sizeof(temp_buffer1)-1));
+    }
+    strncat(temp_buffer1, "\n", (sizeof(temp_buffer1)-2));
+    temp_buffer1[sizeof( temp_buffer1 )-1]='\x0';
+
+    gm_log( GM_LOG_TRACE, "data:\n%s\n", temp_buffer1);
+
+    if(add_job_to_queue( current_client,
+                         mod_gm_opt->server_list,
+                         exec_job->result_queue,
+                         NULL,
+                         temp_buffer1,
+                         GM_JOB_PRIO_NORMAL,
+                         GM_DEFAULT_JOB_RETRIES,
+                         mod_gm_opt->transportmode,
+                         TRUE
+                        ) == GM_OK) {
+        gm_log( GM_LOG_TRACE, "send_result_back() finished successfully\n" );
+    }
+    else {
+        gm_log( GM_LOG_TRACE, "send_result_back() finished unsuccessfully\n" );
+    }
+
+    if( mod_gm_opt->dupserver_num ) {
+        strncpy(temp_buffer2, "type=passive\n", (sizeof(temp_buffer1)-2));
+        strncat(temp_buffer2, temp_buffer1, (sizeof(temp_buffer2)-2));
+        temp_buffer2[sizeof( temp_buffer2 )-1]='\x0';
+        if( add_job_to_queue( current_client_dup,
+                              mod_gm_opt->dupserver_list,
+                              exec_job->result_queue,
+                              NULL,
+                              temp_buffer2,
+                              GM_JOB_PRIO_NORMAL,
+                              GM_DEFAULT_JOB_RETRIES,
+                              mod_gm_opt->transportmode,
+                              TRUE
+                            ) == GM_OK) {
+            gm_log( GM_LOG_TRACE, "send_result_back() finished successfully for duplicate server.\n" );
+        }
+        else {
+            gm_log( GM_LOG_TRACE, "send_result_back() finished unsuccessfully for duplicate server\n" );
+        }
+    }
+    else {
+        gm_log( GM_LOG_TRACE, "send_result_back() has no duplicate servers to send to.\n" );
+    }
+
+    return;
+}
