@@ -47,8 +47,7 @@ static pthread_mutex_t mod_gm_result_list_mutex = PTHREAD_MUTEX_INITIALIZER;
 void *gearman_module_handle=NULL;
 gearman_client_st client;
 
-int send_now               = FALSE;
-int result_threads_running = 0;
+int send_now, result_threads_running;
 pthread_t result_thr[GM_LISTSIZE];
 char target_queue[GM_BUFFERSIZE];
 char temp_buffer[GM_BUFFERSIZE];
@@ -72,6 +71,8 @@ static void move_results_to_core(void);
 int nebmodule_init( int flags, char *args, nebmodule *handle ) {
     int i;
     int broker_option_errors = 0;
+    send_now                 = FALSE;
+    result_threads_running   = 0;
 
     /* save our handle */
     gearman_module_handle=handle;
@@ -88,12 +89,14 @@ int nebmodule_init( int flags, char *args, nebmodule *handle ) {
     set_default_options(mod_gm_opt);
 
     /* parse arguments */
-    read_arguments( args );
-    gm_log( GM_LOG_INFO,  "Version %s\n", GM_VERSION );
-    gm_log( GM_LOG_TRACE, "args: %s\n", args );
+    gm_log( GM_LOG_DEBUG, "Version %s\n", GM_VERSION );
+    gm_log( GM_LOG_DEBUG, "args: %s\n", args );
     gm_log( GM_LOG_TRACE, "nebmodule_init(%i, %i)\n", flags );
     gm_log( GM_LOG_DEBUG, "running on libgearman %s\n", gearman_version() );
 
+    if( read_arguments( args ) == GM_ERROR )
+        return NEB_ERROR;
+
     /* check for minimum eventbroker options */
     if(!(event_broker_options & BROKER_PROGRAM_STATE)) {
         gm_log( GM_LOG_ERROR, "mod_gearman needs BROKER_PROGRAM_STATE (%i) event_broker_options enabled to work\n", BROKER_PROGRAM_STATE );
@@ -154,11 +157,19 @@ int nebmodule_init( int flags, char *args, nebmodule *handle ) {
     neb_register_callback( NEBCALLBACK_TIMED_EVENT_DATA, gearman_module_handle, 0, handle_timed_events );
 
     /* register export callbacks */
-    for(i=0;i<=GM_NEBTYPESSIZE;i++) {
+    for(i=0;i<GM_NEBTYPESSIZE;i++) {
         if(mod_gm_opt->exports[i]->elem_number > 0)
             neb_register_callback( i, gearman_module_handle, 0, handle_export );
     }
 
+    /* log at least one line into the core logfile */
+    if ( mod_gm_opt->logmode != GM_LOG_MODE_CORE ) {
+        int logmode_saved = mod_gm_opt->logmode;
+        mod_gm_opt->logmode = GM_LOG_MODE_CORE;
+        gm_log( GM_LOG_INFO,  "initialized version %s (libgearman %s)\n", GM_VERSION, gearman_version() );
+        mod_gm_opt->logmode = logmode_saved;
+    }
+
     gm_log( GM_LOG_DEBUG, "finished initializing\n" );
 
     return NEB_OK;
@@ -217,7 +228,7 @@ int nebmodule_deinit( int flags, int reason ) {
     }
 
     /* register export callbacks */
-    for(x=0;x<=GM_NEBTYPESSIZE;x++) {
+    for(x=0;x<GM_NEBTYPESSIZE;x++) {
         if(mod_gm_opt->exports[x]->elem_number > 0)
             neb_deregister_callback( x, gearman_module_handle );
     }
@@ -227,14 +238,16 @@ int nebmodule_deinit( int flags, int reason ) {
     gm_log( GM_LOG_DEBUG, "deregistered callbacks\n" );
 
     /* stop result threads */
-    for(x = 0; x < mod_gm_opt->result_workers; x++) {
+    for(x = 0; x < result_threads_running; x++) {
         pthread_cancel(result_thr[x]);
         pthread_join(result_thr[x], NULL);
     }
 
-    /* cleanup client */
+    /* cleanup */
     free_client(&client);
 
+    mod_gm_free_opt(mod_gm_opt);
+
     return NEB_OK;
 }
 
@@ -388,8 +401,7 @@ static int handle_eventhandler( int event_type, void *data ) {
     gm_log( GM_LOG_TRACE, "got eventhandler event: %s\n", ds->command_line );
 
     temp_buffer[0]='\x0';
-    snprintf( temp_buffer,sizeof( temp_buffer )-1,"type=eventhandler\ncommand_line=%s\n\n\n",ds->command_line );
-    temp_buffer[sizeof( temp_buffer )-1]='\x0';
+    snprintf( temp_buffer,GM_BUFFERSIZE-1,"type=eventhandler\ncommand_line=%s\n\n\n",ds->command_line );
 
     if(add_job_to_queue( &client,
                          mod_gm_opt->server_list,
@@ -495,14 +507,13 @@ static int handle_host_check( int event_type, void *data ) {
     gm_log( GM_LOG_TRACE, "cmd_line: %s\n", processed_command );
 
     temp_buffer[0]='\x0';
-    snprintf( temp_buffer,sizeof( temp_buffer )-1,"type=host\nresult_queue=%s\nhost_name=%s\nstart_time=%i.0\ntimeout=%d\ncommand_line=%s\n\n\n",
+    snprintf( temp_buffer,GM_BUFFERSIZE-1,"type=host\nresult_queue=%s\nhost_name=%s\nstart_time=%i.0\ntimeout=%d\ncommand_line=%s\n\n\n",
               mod_gm_opt->result_queue,
               hst->name,
               ( int )hst->next_check,
               host_check_timeout,
               processed_command
             );
-    temp_buffer[sizeof( temp_buffer )-1]='\x0';
 
     if(add_job_to_queue( &client,
                          mod_gm_opt->server_list,
@@ -621,7 +632,7 @@ static int handle_svc_check( int event_type, void *data ) {
     gm_log( GM_LOG_TRACE, "cmd_line: %s\n", processed_command );
 
     temp_buffer[0]='\x0';
-    snprintf( temp_buffer,sizeof( temp_buffer )-1,"type=service\nresult_queue=%s\nhost_name=%s\nservice_description=%s\nstart_time=%i.0\ntimeout=%d\ncommand_line=%s\n\n\n",
+    snprintf( temp_buffer,GM_BUFFERSIZE-1,"type=service\nresult_queue=%s\nhost_name=%s\nservice_description=%s\nstart_time=%i.0\ntimeout=%d\ncommand_line=%s\n\n\n",
               mod_gm_opt->result_queue,
               svcdata->host_name,
               svcdata->service_description,
@@ -629,10 +640,9 @@ static int handle_svc_check( int event_type, void *data ) {
               service_check_timeout,
               processed_command
             );
-    temp_buffer[sizeof( temp_buffer )-1]='\x0';
 
     uniq[0]='\x0';
-    snprintf( uniq,sizeof( temp_buffer )-1,"%s-%s", svcdata->host_name, svcdata->service_description);
+    snprintf( uniq,GM_BUFFERSIZE-1,"%s-%s", svcdata->host_name, svcdata->service_description);
 
     /* execute forced checks with high prio as they are propably user requested */
     if(check_result_info.check_options & CHECK_OPTION_FORCE_EXECUTION)
@@ -783,8 +793,7 @@ static void set_target_queue( host *hst, service *svc ) {
             servicegroup * temp_servicegroup = find_servicegroup( mod_gm_opt->servicegroups_list[x] );
             if ( is_service_member_of_servicegroup( temp_servicegroup,svc )==TRUE ) {
                 gm_log( GM_LOG_TRACE, "service is member of servicegroup: %s\n", mod_gm_opt->servicegroups_list[x] );
-                snprintf( target_queue, sizeof(target_queue)-1, "servicegroup_%s", mod_gm_opt->servicegroups_list[x] );
-                target_queue[sizeof( target_queue )-1]='\x0';
+                snprintf( target_queue, GM_BUFFERSIZE-1, "servicegroup_%s", mod_gm_opt->servicegroups_list[x] );
                 return;
             }
             x++;
@@ -797,8 +806,7 @@ static void set_target_queue( host *hst, service *svc ) {
         hostgroup * temp_hostgroup = find_hostgroup( mod_gm_opt->hostgroups_list[x] );
         if ( is_host_member_of_hostgroup( temp_hostgroup,hst )==TRUE ) {
             gm_log( GM_LOG_TRACE, "server is member of hostgroup: %s\n", mod_gm_opt->hostgroups_list[x] );
-            snprintf( target_queue, sizeof(target_queue)-1, "hostgroup_%s", mod_gm_opt->hostgroups_list[x] );
-            target_queue[sizeof( target_queue )-1]='\x0';
+            snprintf( target_queue, GM_BUFFERSIZE-1, "hostgroup_%s", mod_gm_opt->hostgroups_list[x] );
             return;
         }
         x++;
@@ -807,16 +815,14 @@ static void set_target_queue( host *hst, service *svc ) {
     if ( svc ) {
         /* pass into the general service queue */
         if ( mod_gm_opt->services == GM_ENABLED && svc ) {
-            snprintf( target_queue, sizeof(target_queue)-1, "service" );
-            target_queue[sizeof( target_queue )-1]='\x0';
+            snprintf( target_queue, GM_BUFFERSIZE-1, "service" );
             return;
         }
     }
     else {
         /* pass into the general host queue */
         if ( mod_gm_opt->hosts == GM_ENABLED ) {
-            snprintf( target_queue, sizeof(target_queue)-1, "host" );
-            target_queue[sizeof( target_queue )-1]='\x0';
+            snprintf( target_queue, GM_BUFFERSIZE-1, "host" );
             return;
         }
     }
@@ -827,7 +833,7 @@ static void set_target_queue( host *hst, service *svc ) {
 
 /* start our threads */
 static void start_threads(void) {
-    if ( !result_threads_running ) {
+    if ( result_threads_running < mod_gm_opt->result_workers ) {
         /* create result worker */
         int x;
         for(x = 0; x < mod_gm_opt->result_workers; x++) {
@@ -870,10 +876,10 @@ int handle_perfdata(int event_type, void *data) {
                 }
 
                 uniq[0]='\x0';
-                snprintf( uniq,sizeof( temp_buffer )-1,"%s", hostchkdata->host_name);
+                snprintf( uniq,GM_BUFFERSIZE-1,"%s", hostchkdata->host_name);
 
                 temp_buffer[0]='\x0';
-                snprintf( temp_buffer,sizeof( temp_buffer )-1,
+                snprintf( temp_buffer,GM_BUFFERSIZE-1,
                             "DATATYPE::HOSTPERFDATA\t"
                             "TIMET::%d\t"
                             "HOSTNAME::%s\t"
@@ -885,7 +891,6 @@ int handle_perfdata(int event_type, void *data) {
                             hostchkdata->host_name, hostchkdata->perf_data,
                             hostchkdata->command_name, hostchkdata->command_args,
                             hostchkdata->state, hostchkdata->state_type);
-                temp_buffer[sizeof( temp_buffer )-1]='\x0';
                 has_perfdata = TRUE;
             }
             break;
@@ -906,10 +911,10 @@ int handle_perfdata(int event_type, void *data) {
                 }
 
                 uniq[0]='\x0';
-                snprintf( uniq,sizeof( temp_buffer )-1,"%s-%s", srvchkdata->host_name, srvchkdata->service_description);
+                snprintf( uniq,GM_BUFFERSIZE-1,"%s-%s", srvchkdata->host_name, srvchkdata->service_description);
 
                 temp_buffer[0]='\x0';
-                snprintf( temp_buffer,sizeof( temp_buffer )-1,
+                snprintf( temp_buffer,GM_BUFFERSIZE-1,
                             "DATATYPE::SERVICEPERFDATA\t"
                             "TIMET::%d\t"
                             "HOSTNAME::%s\t"
@@ -922,7 +927,7 @@ int handle_perfdata(int event_type, void *data) {
                             srvchkdata->host_name, srvchkdata->service_description,
                             srvchkdata->perf_data, svc->service_check_command,
                             srvchkdata->state, srvchkdata->state_type);
-                temp_buffer[sizeof( temp_buffer )-1]='\x0';
+                temp_buffer[GM_BUFFERSIZE-1]='\x0';
                 has_perfdata = TRUE;
             }
             break;
@@ -987,7 +992,7 @@ int handle_export(int callback_type, void *data) {
         case NEBCALLBACK_PROCESS_DATA:                      /*  7 */
             npd    = (nebstruct_process_data *)data;
             type   = nebtype2str(npd->type);
-            snprintf( temp_buffer,sizeof( temp_buffer )-1, "{\"callback_type\":\"%s\",\"type\":\"%s\",\"flags\":%d,\"attr\":%d,\"timestamp\":%d.%d}",
+            snprintf( temp_buffer,GM_BUFFERSIZE-1, "{\"callback_type\":\"%s\",\"type\":\"%s\",\"flags\":%d,\"attr\":%d,\"timestamp\":%d.%d}",
                     "NEBCALLBACK_PROCESS_DATA",
                     type,
                     npd->flags,
@@ -1000,7 +1005,7 @@ int handle_export(int callback_type, void *data) {
             nted       = (nebstruct_timed_event_data *)data;
             event_type = eventtype2str(nted->event_type);
             type       = nebtype2str(nted->type);
-            snprintf( temp_buffer,sizeof( temp_buffer )-1, "{\"callback_type\":\"%s\",\"event_type\":\"%s\",\"type\":\"%s\",\"flags\":%d,\"attr\":%d,\"timestamp\":%d.%d,\"recurring\":%d,\"run_time\":%d}",
+            snprintf( temp_buffer,GM_BUFFERSIZE-1, "{\"callback_type\":\"%s\",\"event_type\":\"%s\",\"type\":\"%s\",\"flags\":%d,\"attr\":%d,\"timestamp\":%d.%d,\"recurring\":%d,\"run_time\":%d}",
                     "NEBCALLBACK_TIMED_EVENT_DATA",
                     event_type,
                     type,
@@ -1017,7 +1022,7 @@ int handle_export(int callback_type, void *data) {
             nld    = (nebstruct_log_data *)data;
             buffer = escapestring(nld->data);
             type   = nebtype2str(nld->type);
-            snprintf( temp_buffer,sizeof( temp_buffer )-1, "{\"callback_type\":\"%s\",\"type\":\"%s\",\"flags\":%d,\"attr\":%d,\"timestamp\":%d.%d,\"entry_time\":%d,\"data_type\":%d,\"data\":\"%s\"}",
+            snprintf( temp_buffer,GM_BUFFERSIZE-1, "{\"callback_type\":\"%s\",\"type\":\"%s\",\"flags\":%d,\"attr\":%d,\"timestamp\":%d.%d,\"entry_time\":%d,\"data_type\":%d,\"data\":\"%s\"}",
                     "NEBCALLBACK_LOG_DATA",
                     type,
                     nld->flags,
@@ -1081,7 +1086,6 @@ int handle_export(int callback_type, void *data) {
             return 0;
     }
 
-    temp_buffer[sizeof( temp_buffer )-1]='\x0';
     if(temp_buffer[0] != '\x0') {
 
         for(i=0;i<mod_gm_opt->exports[callback_type]->elem_number;i++) {
