@@ -17,8 +17,18 @@ void printf_hex(char* text, int length) {
     return;
 }
 
+mod_gm_opt_t * renew_opts(void);
+mod_gm_opt_t * renew_opts() {
+    mod_gm_opt_t *mod_gm_opt;
+
+    mod_gm_opt = malloc(sizeof(mod_gm_opt_t));
+    set_default_options(mod_gm_opt);
+
+    return mod_gm_opt;
+}
+
 int main(void) {
-    plan(44);
+    plan(58);
 
     /* lowercase */
     char test[100];
@@ -28,6 +38,14 @@ int main(void) {
     strcpy(test, "yeS"); like(lc(test), "yes", "lc(yeS)");
 
 
+    /* trim */
+    strcpy(test, "    text  "); like(ltrim(test), "text  ",   "ltrim()");
+    strcpy(test, "    text  "); like(rtrim(test), "    text", "rtrim()");
+    strcpy(test, "    text  "); like(trim(test),  "text",     "trim()");
+    char *test2;
+    test2 = strdup("   text   ");  like(trim(test2),  "text", "trim()");
+    free(test2);
+
     /* parse_yes_or_no */
     ok(parse_yes_or_no(NULL,    GM_ENABLED)  == GM_ENABLED, "parse_yes_or_no 1");
     ok(parse_yes_or_no(NULL,    GM_DISABLED) == GM_DISABLED, "parse_yes_or_no 2");
@@ -46,14 +64,16 @@ int main(void) {
 
     /* trim */
     ok(trim(NULL) == NULL, "trim(NULL)");
-    strcpy(test, " test "); like(trim(test), "test", "trim(' test ')");
+    strcpy(test, " test "); like(trim(test), "^test$", "trim(' test ')");
+    strcpy(test, "\ntest\n"); like(trim(test), "^test$", "trim('\\ntest\\n')");
 
     /* reading keys */
     mod_gm_opt_t *mod_gm_opt;
     mod_gm_opt = malloc(sizeof(mod_gm_opt_t));
     int rc = set_default_options(mod_gm_opt);
+
     ok(rc == 0, "setting default options");
-    mod_gm_opt->keyfile = "t/data/test1.key";
+    mod_gm_opt->keyfile = strdup("t/data/test1.key");
     read_keyfile(mod_gm_opt);
     //printf_hex(mod_gm_opt->crypt_key, 32);
     test[0]='\x0';
@@ -66,12 +86,14 @@ int main(void) {
     }
     like(test, "3131313131313131313131313131313131313131313131313131313131310000", "read keyfile t/data/test1.key");
 
-    mod_gm_opt->keyfile = "t/data/test2.key";
+    free(mod_gm_opt->keyfile);
+    mod_gm_opt->keyfile = strdup("t/data/test2.key");
     read_keyfile(mod_gm_opt);
 
     like(mod_gm_opt->crypt_key, "abcdef", "reading keyfile t/data/test2.key");
 
-    mod_gm_opt->keyfile = "t/data/test3.key";
+    free(mod_gm_opt->keyfile);
+    mod_gm_opt->keyfile = strdup("t/data/test3.key");
     read_keyfile(mod_gm_opt);
     //printf_hex(mod_gm_opt->crypt_key, 32);
     like(mod_gm_opt->crypt_key, "11111111111111111111111111111111", "reading keyfile t/data/test3.key");
@@ -143,13 +165,46 @@ int main(void) {
     ok(t.tv_usec == 0, "string2timeval 8");
 
     /* command line parsing */
+    mod_gm_free_opt(mod_gm_opt);
+    mod_gm_opt = renew_opts();
     strcpy(test, "server=host:4730");
     parse_args_line(mod_gm_opt, test, 0);
     like(mod_gm_opt->server_list[0], "host:4730", "server=host:4730");
+    ok(mod_gm_opt->server_num == 1, "server_number = %d", mod_gm_opt->server_num);
 
+    mod_gm_free_opt(mod_gm_opt);
+    mod_gm_opt = renew_opts();
     strcpy(test, "server=:4730");
     parse_args_line(mod_gm_opt, test, 0);
-    like(mod_gm_opt->server_list[1], "localhost:4730", "server=:4730");
+    like(mod_gm_opt->server_list[0], "localhost:4730", "server=:4730");
+    ok(mod_gm_opt->server_num == 1, "server_number = %d", mod_gm_opt->server_num);
+
+    mod_gm_free_opt(mod_gm_opt);
+    mod_gm_opt = renew_opts();
+    strcpy(test, "server=localhost:4730");
+    parse_args_line(mod_gm_opt, test, 0);
+    strcpy(test, "server=localhost:4730");
+    parse_args_line(mod_gm_opt, test, 0);
+    like(mod_gm_opt->server_list[0], "localhost:4730", "duplicate server");
+    ok(mod_gm_opt->server_num == 1, "server_number = %d", mod_gm_opt->server_num);
+
+    mod_gm_free_opt(mod_gm_opt);
+    mod_gm_opt = renew_opts();
+    strcpy(test, "server=localhost:4730,localhost:4730,:4730,host:4730,");
+    parse_args_line(mod_gm_opt, test, 0);
+    like(mod_gm_opt->server_list[0], "localhost:4730", "duplicate server");
+    like(mod_gm_opt->server_list[1], "host:4730", "duplicate server");
+    ok(mod_gm_opt->server_num == 2, "server_number = %d", mod_gm_opt->server_num);
+
+    /* escape newlines */
+    char * escaped = gm_escape_newlines(" test\n", GM_DISABLED);
+    is(escaped, " test\\n", "untrimmed escape string");
+    free(escaped);
+    escaped = gm_escape_newlines(" test\n", GM_ENABLED);
+    is(escaped, "test", "trimmed escape string");
+    free(escaped);
+
+    mod_gm_free_opt(mod_gm_opt);
 
     return exit_status();
 }
